# -*- coding: UTF-8 -*-

# Author: Fabian Steiner, <info@fabis-site.net>

import re
import subprocess

def get_all_input_devices():
    """This function parses the /proc/bus/input/devices file in order to 
    generate a list with all known input devices. Every input device will 
    be represented by a python dictionary with "phys" and "handler" 
    keys. 
    """
    input_devices = [] # list that will hold the input devices
    device = {}
    regex = re.compile(r'event[0-9]+')
    f = file('/proc/bus/input/devices')
    for line in f:
        if line == '\n':
            # we have got a new device here
            if device:
                input_devices.append(device)
            device = {}
        else:
            # we are trying to extract the phys path of the device
            if line.startswith('P: Phys=') :
                device['phys'] = line.split('=')[1].strip()
            # we are trying to get the corresponding event device
            # (event0, event1, etc.)
            elif line.startswith('H: Handlers='):
                device['handler'] ='/dev/input/%s' % regex.search(line).group()
    f.close()
    
    return input_devices


def get_video_cards():
    lines = []
    cards = []
    p1 = subprocess.Popen(['lspci', '-n'], stdout=subprocess.PIPE)
    p2 = subprocess.Popen(['grep', '0300'], stdin=p1.stdout, stdout=subprocess.PIPE)
    lines = p2.communicate()[0].split('\n')
    if not lines:
        return []
    for line in lines[:-1]:
        busid = line.split(' ')[0].strip()
        busid_parts = tuple(map(lambda x: str(int(x, 16)), re.compile(r'[:\.]').split(busid)))[-3:]
        cards.append(busid_parts)
    cards.sort()
    return map(_busid_repr, cards)


def _busid_repr(busid):
    return 'PCI:' + ':'.join(busid)
    
__all__ = ['get_all_input_devices', 'get_video_cards']
